#include <math.h>
#include "radialanimation.h"

namespace PixelMaestro {
	RadialAnimation::RadialAnimation(Section& section) : Animation(section) {
		type_ = AnimationType::Radial;
		map();
	}

	/**
	 * Returns the resolution of each radial.
	 * @return Radial resolution.
	 */
	uint8_t RadialAnimation::get_resolution() const {
		return resolution_;
	}

	/**
	 * Updates the map.
	 * This only occurs when the grid size or orientation changes.
	 */
	void RadialAnimation::map() {
		Point center = get_center();
		if (orientation_ == Orientation::Vertical || orientation_ == Orientation::VerticalFlipped) {
			// For each Pixel, calculate the slope from the center.
			for (uint16_t y = 0; y < dimensions_.y; y++) {
				for (uint16_t x = 0; x < dimensions_.x; x++) {
					if (x == center.x || y == center.y) {
						set_map_color_index(x, y, 0);
					}
					else {
						set_map_color_index(x, y, static_cast<uint16_t>(((y - center.y) / (float)(x - center.x)) * resolution_) % 255);
					}
				}
			}
		}
		else {	// Horizontal
			// For each Pixel, calculate its distance from the center of the grid, then use the distance to choose the index of the correct color.
			for (uint16_t y = 0; y < dimensions_.y; y++) {
				uint16_t y_squared_ = pow(y - center.y, 2);
				for (uint16_t x = 0; x < dimensions_.x; x++) {
					set_map_color_index(x, y, sqrt(pow(x - center.x, 2) + y_squared_));
				}
			}
		}
	}

	/**
	 * Changes the size of each spoke (vertical mode only).
	 * @param resolution New resolution.
	 */
	void RadialAnimation::set_resolution(uint8_t resolution) {
		this->resolution_ = resolution;
		map();
	}

	void RadialAnimation::update() {
		update_cycle(0, palette_->get_num_colors());
	}
}
